@@ -39,3 +39,17 @@ WS_ON_CLOSE = "ws_on_close"
 OFFLINE_INVALID_WS_MSG = "offline_invalid_ws_msg"
 
 OFFLINE_INVALID_UPLOAD_MSG = "offline_invalid_upload_msg"
+
+OFFLINE_INVALID_METRIC_MSG = "offline_invalid_metric_msg"
+
+OFFLINE_INVALID_PARAMETER_MSG = "offline_invalid_parameter_msg"
+
+OFFLINE_INVALID_OS_PACKAGES_MSG = "offline_invalid_os_packages_msg"
+
+OFFLINE_INVALID_GRAPH_MSG = "offline_invalid_graph_msg"
+
+OFFLINE_INVALID_SYSTEM_DETAILS_MSG = "offline_invalid_system_details_msg"
+
+OFFLINE_INVALID_CLOUD_DETAILS_MSG = "offline_invalid_cloud_details_msg"
+
+OFFLINE_INVALID_FILE_NAME_MSG = "offline_invalid_file_name_msg"
