@@ -18,6 +18,7 @@ import six
 
 from .._typing import (
     Any,
+    Dict,
     Generator,
     Iterator,
     List,
@@ -26,13 +27,148 @@ from .._typing import (
     TensorflowInput,
     Tuple,
 )
+from ..data_structure import Histogram
 from ..experiment import BaseExperiment
 from ..logging_messages import TF_KERAS_CALLBACK_WARNING_CLOSED_EXPERIMENT
-from ..predictor import Predictor
-from ..utils import Histogram, get_time_monotonic, tensor_length
+from ..utils import get_time_monotonic, tensor_length
 
 LOGGER = logging.getLogger(__name__)
 
+XGBOOST_PARAMS_EXCLUDE = {"model", "evaluation_result_list", "iteration", "handle"}
+XGBOOST_BOOSTER_ATTRS_EXCLUDE = {"handle"}
+
+
+def _filter_xgboost_attributes(obj, exclude_set):
+    # type: (Any, Set[int]) -> Generator[Tuple[str, Any], None, None]
+    for attribute_name in dir(obj):
+        if attribute_name.startswith("_"):
+            continue
+
+        if attribute_name in exclude_set:
+            continue
+
+        attribute_value = getattr(obj, attribute_name)
+
+        if callable(attribute_value):
+            continue
+
+        if attribute_value is None:
+            continue
+
+        yield (attribute_name, attribute_value)
+
+
+def _get_xgboost_env_params(env):
+    # type: (Any) -> Generator[Tuple[str, Any], None, None]
+    for attribute in _filter_xgboost_attributes(env, XGBOOST_PARAMS_EXCLUDE):
+        yield attribute
+
+
+def _get_xgboost_env_model_params(booster):
+    # type: (Any) -> Generator[Tuple[str, Any], None, None]
+    for attribute in _filter_xgboost_attributes(booster, XGBOOST_BOOSTER_ATTRS_EXCLUDE):
+        yield attribute
+
+
+def _get_xgboost_booster_graph(booster):
+    # type: (Any) -> str
+    from xgboost import to_graphviz
+
+    graphviz_source = to_graphviz(booster).source  # type: str
+    return graphviz_source
+
+
+# TODO:
+# Find a better name than obj for the env/model parameter.
+
+
+def _log_xgboost_step(experiment, xgboost_step):
+    # type: (BaseExperiment, int) -> None
+    """
+    Logs the step of the training to the experiment
+    """
+    try:
+        experiment.set_step(xgboost_step)
+    except Exception:
+        LOGGER.debug("Failed to log the XGBoost step", exc_info=True)
+
+
+def _log_xgboost_parameters(experiment, obj):
+    # type: (BaseExperiment, Any) -> None
+    """
+    Logs the parameters of the training to the experiment
+    """
+    try:
+        if experiment.auto_param_logging:
+            if not experiment._storage["xgboost"]["env_parameter_set"]:
+                for attribute_name, attribute_value in _get_xgboost_env_params(obj):
+                    experiment._log_parameter(
+                        attribute_name, attribute_value, framework="xgboost"
+                    )
+
+                # Set only once the parameters
+                experiment._storage["xgboost"]["env_parameter_set"] = True
+    except Exception:
+        LOGGER.debug("Failed to log the XGBoost params", exc_info=True)
+
+
+def _log_xgboost_model_attributes(experiment, booster):
+    # type: (BaseExperiment, Any) -> None
+    """
+    Logs the attributes of the model to the experiment
+    """
+    try:
+        if experiment.auto_param_logging:
+            if not experiment._storage["xgboost"]["env_model_parameter_set"]:
+
+                for attribute_name, attribute_value in _get_xgboost_env_model_params(
+                    booster
+                ):
+                    experiment._log_parameter(
+                        attribute_name, attribute_value, framework="xgboost"
+                    )
+
+                # Set only once the model parameters
+                experiment._storage["xgboost"]["env_model_parameter_set"] = True
+    except Exception:
+        LOGGER.debug("Failed to log the XGBoost booster attributes", exc_info=True)
+
+
+def _log_xgboost_model_metrics(experiment, evals_log):
+    # type: (BaseExperiment, Dict) -> None
+    """
+    Logs the metrics to the experiment
+    """
+    try:
+        if experiment.auto_metric_logging:
+            xgboost_metrics = evals_log
+            for context, metrics in xgboost_metrics.items():
+                with experiment.context_manager(context):
+                    experiment._log_metrics(metrics, framework="xgboost")
+    except Exception:
+        LOGGER.debug("Failed to log the XGBoost metrics", exc_info=True)
+
+
+def _log_xgboost_model_graph(experiment, booster):
+    # type: (BaseExperiment, Any) -> None
+    """
+    Logs the graph of the model to the experiment
+    """
+    try:
+        if experiment.log_graph:
+            if not experiment._storage["xgboost"]["model_graph_set"]:
+
+                booster_graph = _get_xgboost_booster_graph(booster)
+
+                experiment._set_model_graph(booster_graph, framework="xgboost")
+
+                experiment._storage["xgboost"]["model_graph_set"] = True
+    # xgboost.to_graphviz can raises ImportError if optional dependencies are not installed
+    except ImportError as exc:
+        experiment._log_once_at_level(logging.WARNING, str(exc), exc_info=True)
+    except Exception:
+        LOGGER.debug("Failed to log the XGBoost metrics", exc_info=True)
+
 
 def get_standardized_layer_set(layer_list_raw, layer_names):
     # type: (List[str], List[str]) -> Set[str]
@@ -313,13 +449,13 @@ def get_tensorflow_activation_histogram_all(model, output_tensor, inputs, batch_
 
 
 def build_base_callback(base):
-    class BaseCallback(base):  # type: ignore
+    class CometBaseKerasCallback(base):  # type: ignore
         """
         Base Keras callback.
         """
 
         def __init__(self):
-            super(BaseCallback, self).__init__()
+            super(CometBaseKerasCallback, self).__init__()
 
         def on_epoch_begin(self, epoch, logs=None):
             pass
@@ -369,45 +505,20 @@ def build_base_callback(base):
         def on_predict_batch_end(self, batch, logs=None):
             pass
 
-    return BaseCallback
-
-
-def build_predictive_early_stopping_keras_callback(base):
-    class PredictiveEarlyStoppingKerasCallback(base):  # type: ignore
-        def __init__(self, experiment, predictor, predict_epoch_rate=1):
-            # type: (BaseExperiment, Predictor, int) -> None
-            super(PredictiveEarlyStoppingKerasCallback, self).__init__()
-            self.predictor = predictor
-            self.predict_epoch_rate = predict_epoch_rate
-            self.experiment = experiment
-
-        def on_epoch_end(self, epoch, logs={}):
-            if check_experiment_closed_and_warn(
-                self.experiment, callback_method_name="on_epoch_end"
-            ):
-                return
-
-            if epoch % self.predict_epoch_rate == 0:
-                loss = logs.get(self.predictor.loss_name)
-                self.predictor.report_loss(loss)
-
-                if self.predictor.stop_early(epoch=epoch):
-                    self.model.stop_training = True
-
-    return PredictiveEarlyStoppingKerasCallback
+    return CometBaseKerasCallback
 
 
 def build_empty_keras_callback(base):
-    class EmptyKerasCallback(base):  # type: ignore
+    class CometEmptyKerasCallback(base):  # type: ignore
         """
         Empty Keras callback.
         """
 
-    return EmptyKerasCallback
+    return CometEmptyKerasCallback
 
 
 def build_keras_callback(base):
-    class KerasCallback(base):  # type: ignore
+    class CometKerasCallback(base):  # type: ignore
         """Keras callback to report params, metrics to Comet.ml Experiment()"""
 
         def __init__(
@@ -425,7 +536,7 @@ def build_keras_callback(base):
             Create a new experiment and submit source code.
             :param api_key: User's API key. Required.
             """
-            super(KerasCallback, self).__init__()
+            super(CometKerasCallback, self).__init__()
             self.inputs = inputs
             self.targets = targets
 
@@ -433,22 +544,22 @@ def build_keras_callback(base):
             if log_params is not None:
                 experiment._log_once_at_level(
                     logging.INFO,
-                    "Passing log_params to KerasCallback is deprecated; use experiment.auto_param_logging",
+                    "Passing log_params to CometKerasCallback is deprecated; use experiment.auto_param_logging",
                 )
             if log_metrics is not None:
                 experiment._log_once_at_level(
                     logging.INFO,
-                    "Passing log_metrics to KerasCallback is deprecated; use experiment.auto_metric_logging",
+                    "Passing log_metrics to CometKerasCallback is deprecated; use experiment.auto_metric_logging",
                 )
             if log_graph is not None:
                 experiment._log_once_at_level(
                     logging.INFO,
-                    "Passing log_graph to KerasCallback is deprecated; use experiment.log_graph",
+                    "Passing log_graph to CometKerasCallback is deprecated; use experiment.log_graph",
                 )
             if log_histograms is not None:
                 experiment._log_once_at_level(
                     logging.INFO,
-                    "Passing log_histograms to KerasCallback is deprecated; use experiment.auto_histogram_*_logging",
+                    "Passing log_histograms to CometKerasCallback is deprecated; use experiment.auto_histogram_*_logging",
                 )
 
             # Inits the experiment with reference to the name of this class. Required for loading the correct
@@ -886,7 +997,7 @@ def build_keras_callback(base):
                     exc_info=True,
                 )
 
-    return KerasCallback
+    return CometKerasCallback
 
 
 def get_keras_model(experiment, model):
