@@ -15,12 +15,7 @@
 import logging
 
 from ..logging_messages import TF_KERAS_FALLBACK_FAILED
-from ._base import (
-    build_base_callback,
-    build_empty_keras_callback,
-    build_keras_callback,
-    build_predictive_early_stopping_keras_callback,
-)
+from ._base import build_base_callback, build_empty_keras_callback, build_keras_callback
 
 LOGGER = logging.getLogger(__name__)
 
@@ -38,12 +33,8 @@ except ImportError as exc:
 
 baseclass = keras.callbacks.Callback
 
-BaseCallback = build_base_callback(baseclass)
+CometBaseKerasCallback = build_base_callback(baseclass)
 
-PredictiveEarlyStoppingKerasCallback = build_predictive_early_stopping_keras_callback(
-    BaseCallback
-)
+CometEmptyKerasCallback = build_empty_keras_callback(CometBaseKerasCallback)
 
-EmptyKerasCallback = build_empty_keras_callback(BaseCallback)
-
-KerasCallback = build_keras_callback(BaseCallback)
+CometKerasCallback = build_keras_callback(CometBaseKerasCallback)
