@@ -14,19 +14,10 @@
 
 import tensorflow as tf
 
-from ._base import (
-    build_base_callback,
-    build_empty_keras_callback,
-    build_keras_callback,
-    build_predictive_early_stopping_keras_callback,
-)
+from ._base import build_base_callback, build_empty_keras_callback, build_keras_callback
 
-BaseCallback = build_base_callback(tf.keras.callbacks.Callback)
+CometBaseKerasCallback = build_base_callback(tf.keras.callbacks.Callback)
 
-PredictiveEarlyStoppingKerasCallback = build_predictive_early_stopping_keras_callback(
-    BaseCallback
-)
+CometEmptyKerasCallback = build_empty_keras_callback(CometBaseKerasCallback)
 
-EmptyKerasCallback = build_empty_keras_callback(BaseCallback)
-
-KerasCallback = build_keras_callback(BaseCallback)
+CometKerasCallback = build_keras_callback(CometBaseKerasCallback)
