@@ -32,58 +32,65 @@ from six.moves.queue import Empty, Queue
 from six.moves.urllib.parse import urlencode, urlsplit, urlunsplit
 
 from ._reporting import ON_EXIT_DIDNT_FINISH_UPLOAD_SDK
-from ._typing import Any, Dict, List, Optional, Tuple
-from .batch_utils import ParametersBatch
+from ._typing import Any, Dict, List, Optional, Tuple, Union
+from .batch_utils import MessageBatch, MessageBatchItem, ParametersBatch
 from .compat_utils import json_dump
 from .config import (
     ADDITIONAL_STREAMER_UPLOAD_TIMEOUT,
     DEFAULT_FILE_UPLOAD_READ_TIMEOUT,
     DEFAULT_PARAMETERS_UPDATE_INTERVAL_SECONDS,
     DEFAULT_STREAMER_MSG_TIMEOUT,
+    DEFAULT_WAIT_FOR_FINISH_SLEEP_INTERVAL,
+    MESSAGE_BATCH_METRIC_INTERVAL_SECONDS,
+    MESSAGE_BATCH_METRIC_MAX_BATCH_SIZE,
+    MESSAGE_BATCH_USE_COMPRESSION_DEFAULT,
+    OFFLINE_EXPERIMENT_MESSAGES_JSON_FILE_NAME,
 )
 from .connection import (
     FileUploadManager,
     FileUploadManagerMonitor,
+    RestServerConnection,
     format_messages_for_ws,
 )
+from .convert_utils import data_to_fp
 from .exceptions import CometRestApiException
 from .file_uploader import is_user_text
 from .json_encoder import NestedEncoder
 from .logging_messages import (
     CLOUD_DETAILS_MSG_SENDING_ERROR,
     FILE_UPLOADS_PROMPT,
+    FILENAME_DETAILS_MSG_SENDING_ERROR,
+    METRICS_BATCH_MSG_SENDING_ERROR,
     MODEL_GRAPH_MSG_SENDING_ERROR,
     OS_PACKAGE_MSG_SENDING_ERROR,
     STREAMER_CLOSED_PUT_MESSAGE_FAILED,
     STREAMER_FAILED_TO_PROCESS_ALL_MESSAGES,
     STREAMER_WAIT_FOR_FINISH_FAILED,
+    SYSTEM_DETAILS_MSG_SENDING_ERROR,
     WAITING_DATA_UPLOADED,
 )
 from .messages import (
     BaseMessage,
     CloseMessage,
     CloudDetailsMessage,
-    Message,
+    FileNameMessage,
+    MetricMessage,
     ModelGraphMessage,
     OsPackagesMessage,
+    ParameterMessage,
     RemoteAssetMessage,
     SystemDetailsMessage,
     UploadFileMessage,
     UploadInMemoryMessage,
+    WebSocketMessage,
 )
-from .utils import (
-    data_to_fp,
-    log_once_at_level,
-    wait_for_done,
-    write_file_like_to_tmp_file,
-)
+from .utils import log_once_at_level, wait_for_done, write_file_like_to_tmp_file
 
 DEBUG = False
 LOGGER = logging.getLogger(__name__)
 
 
 class BaseStreamer(threading.Thread):
-
     __metaclass__ = abc.ABCMeta
 
     def __init__(self, initial_offset, queue_timeout):
@@ -199,24 +206,31 @@ class Streamer(BaseStreamer):
         self,
         ws_connection,
         beat_duration,
-        connection,
+        connection,  # type: RestServerConnection
         initial_offset,
         experiment_key,
         api_key,
         run_id,
         project_id,
         rest_api_client,
-        worker_cpu_ratio,
+        worker_cpu_ratio,  # type: int
+        worker_count,  # type: Optional[int]
+        verify_tls,  # type: bool
         pending_rpcs_callback=None,
         msg_waiting_timeout=DEFAULT_STREAMER_MSG_TIMEOUT,
         file_upload_waiting_timeout=ADDITIONAL_STREAMER_UPLOAD_TIMEOUT,
         file_upload_read_timeout=DEFAULT_FILE_UPLOAD_READ_TIMEOUT,
-        wait_for_finish_sleep_interval=15,
+        wait_for_finish_sleep_interval=DEFAULT_WAIT_FOR_FINISH_SLEEP_INTERVAL,
         parameters_batch_base_interval=DEFAULT_PARAMETERS_UPDATE_INTERVAL_SECONDS,
+        message_batch_enabled=False,
+        message_batch_metric_compress=MESSAGE_BATCH_USE_COMPRESSION_DEFAULT,
+        message_batch_metric_interval=MESSAGE_BATCH_METRIC_INTERVAL_SECONDS,
+        message_batch_metric_max_size=MESSAGE_BATCH_METRIC_MAX_BATCH_SIZE,
     ):
+        # type: (...) -> None
         super(Streamer, self).__init__(initial_offset, beat_duration / 1000.0)
         self.daemon = True
-        self.name = "Streamer(%r)" % (ws_connection)
+        self.name = "Streamer(%r)" % ws_connection
         self.ws_connection = ws_connection
         self.connection = connection
         self.rest_api_client = rest_api_client
@@ -233,16 +247,31 @@ class Streamer(BaseStreamer):
         self.file_upload_waiting_timeout = file_upload_waiting_timeout
         self.file_upload_read_timeout = file_upload_read_timeout
 
-        self.file_upload_manager = FileUploadManager(worker_cpu_ratio)
+        self.file_upload_manager = FileUploadManager(worker_cpu_ratio, worker_count)
 
         self.experiment_key = experiment_key
         self.api_key = api_key
         self.run_id = run_id
         self.project_id = project_id
 
+        self.verify_tls = verify_tls
+
         self.parameters_batch = ParametersBatch(parameters_batch_base_interval)
 
+        self.message_batch_enabled = message_batch_enabled
+        self.message_batch_metric_compress = message_batch_metric_compress
+        self.message_batch_metrics = MessageBatch(
+            base_interval=message_batch_metric_interval,
+            max_size=message_batch_metric_max_size,
+        )
+
         LOGGER.debug("Streamer instantiated with ws url %s", self.ws_connection)
+        LOGGER.debug(
+            "Message batching enabled: %s, metric batch size: %d, metrics batch interval: %s seconds",
+            message_batch_enabled,
+            message_batch_metric_max_size,
+            message_batch_metric_interval,
+        )
 
     def _before_run(self):
         self.ws_connection.wait_for_connection()
@@ -269,35 +298,50 @@ class Streamer(BaseStreamer):
                     for (message, offset) in messages:
                         if isinstance(message, CloseMessage):
                             return message
-                        elif isinstance(message, UploadFileMessage):
-                            self._process_upload_message(message)
-                        elif isinstance(message, UploadInMemoryMessage):
-                            self._process_upload_in_memory_message(message)
-                        elif isinstance(message, RemoteAssetMessage):
-                            self._process_upload_remote_asset_message(message)
-                        elif isinstance(message, Message):
-                            self._process_ws_message(message, offset)
-                        elif isinstance(message, OsPackagesMessage):
-                            self._process_os_package_message(message)
-                        elif isinstance(message, ModelGraphMessage):
-                            self._process_model_graph_message(message)
-                        elif isinstance(message, SystemDetailsMessage):
-                            self._process_system_details_message(message)
-                        elif isinstance(message, CloudDetailsMessage):
-                            self._process_cloud_details_message(message)
-                        else:
+
+                        message_handlers = {
+                            UploadFileMessage: self._process_upload_message,
+                            UploadInMemoryMessage: self._process_upload_in_memory_message,
+                            RemoteAssetMessage: self._process_upload_remote_asset_message,
+                            WebSocketMessage: self._process_ws_message,
+                            MetricMessage: self._process_metric_message,
+                            ParameterMessage: self._process_parameter_message,
+                            OsPackagesMessage: self._process_os_package_message,
+                            ModelGraphMessage: self._process_model_graph_message,
+                            SystemDetailsMessage: self._process_system_details_message,
+                            CloudDetailsMessage: self._process_cloud_details_message,
+                            FileNameMessage: self._process_file_name_message,
+                        }
+
+                        handler = message_handlers.get(type(message))
+                        if handler is None:
                             raise ValueError("Unknown message type %r", message)
+                        if isinstance(
+                            message, (WebSocketMessage, ParameterMessage, MetricMessage)
+                        ):
+                            handler(message, offset)
+                        else:
+                            handler(message)
 
                 # attempt to send collected parameters
                 if self.parameters_batch.accept(self._send_ws_message):
                     LOGGER.debug("Parameters batch was sent")
 
+                # attempt to send batched metrics
+                if self.message_batch_enabled:
+                    if self.message_batch_metrics.accept(
+                        self._send_metric_messages_batch
+                    ):
+                        LOGGER.debug("Metrics batch was sent")
+
             else:
                 LOGGER.debug("WS connection not ready")
                 # Basic backoff
                 time.sleep(0.5)
         except Exception:
             LOGGER.debug("Unknown streaming error", exc_info=True)
+            # report experiment error
+            self._report_experiment_error()
 
     def _process_upload_message(self, message):
         # type: (UploadFileMessage) -> None
@@ -314,6 +358,7 @@ class Streamer(BaseStreamer):
             metadata=message.metadata,
             project_id=self.project_id,
             timeout=self.file_upload_read_timeout,
+            verify_tls=self.verify_tls,
             upload_endpoint=url,
             on_asset_upload=message._on_asset_upload,
             on_failed_asset_upload=message._on_failed_asset_upload,
@@ -335,6 +380,7 @@ class Streamer(BaseStreamer):
             metadata=message.metadata,
             project_id=self.project_id,
             timeout=self.file_upload_read_timeout,
+            verify_tls=self.verify_tls,
             upload_endpoint=url,
             on_asset_upload=message._on_asset_upload,
             on_failed_asset_upload=message._on_failed_asset_upload,
@@ -356,6 +402,7 @@ class Streamer(BaseStreamer):
             project_id=self.project_id,
             remote_uri=message.remote_uri,
             timeout=self.file_upload_read_timeout,
+            verify_tls=self.verify_tls,
             upload_endpoint=url,
             on_asset_upload=message._on_asset_upload,
             on_failed_asset_upload=message._on_failed_asset_upload,
@@ -363,24 +410,61 @@ class Streamer(BaseStreamer):
         )
         LOGGER.debug("Processing remote uploading message done")
 
+    def _send_metric_messages_batch(self, message_items):
+        # type: (List[MessageBatchItem]) -> None
+        try:
+            self.connection.log_metrics_batch(
+                metrics=message_items, compress=self.message_batch_metric_compress
+            )
+        except CometRestApiException as exc:
+            LOGGER.warning(
+                METRICS_BATCH_MSG_SENDING_ERROR,
+                exc.response.status_code,
+                exc.response.content,
+            )
+            # report experiment error
+            self._report_experiment_error()
+            raise exc
+        except Exception as ex:
+            LOGGER.warning(
+                "Error sending metrics batch (online experiment): %s", ex, exc_info=True
+            )
+            # report experiment error
+            self._report_experiment_error()
+            raise ex
+
     def _send_ws_message(self, message, offset):
-        # type: (Message, int) -> None
-        """The method to be used as callback for parameters batching processing"""
+        # type: (Union[WebSocketMessage, ParameterMessage], int) -> None
+        """The method to be used as callback for parameters batch processing or to send WS messages immediately"""
         try:
             data = self._serialise_message_for_ws(message=message, offset=offset)
             self.ws_connection.send(data)
         except Exception:
             LOGGER.debug("WS sending error", exc_info=True)
+            # report experiment error
+            self._report_experiment_error()
+
+    def _process_parameter_message(self, message, offset):
+        # type: (ParameterMessage, int) -> None
+        # add message to the parameters batch
+        if not self.parameters_batch.append(message=message, offset=offset):
+            LOGGER.debug("Failed to add message to the parameters batch: %r" % message)
+            # report experiment error
+            self._report_experiment_error()
+
+    def _process_metric_message(self, message, offset):
+        # type: (MetricMessage, int) -> None
+        if self.message_batch_enabled:
+            self.message_batch_metrics.append(message=message, offset=offset)
+        else:
+            self._process_ws_message(message, offset)
 
     def _process_ws_message(self, message, offset):
-        # type: (Message, int) -> None
-        batched = self.parameters_batch.append(message=message, offset=offset)
-        if not batched:
-            # send message immediately
-            self._send_ws_message(message, offset)
+        # type: (Union[WebSocketMessage, MetricMessage], int) -> None
+        self._send_ws_message(message, offset)
 
     def _serialise_message_for_ws(self, message, offset):
-        # type: (Message, int) -> str
+        # type: (BaseMessage, int) -> str
         """Enhance provided message with relevant meta-data and serialize it to JSON compatible with WS format"""
         message_dict = message._non_null_dict()
 
@@ -405,8 +489,12 @@ class Streamer(BaseStreamer):
                 exc.response.status_code,
                 exc.response.content,
             )
+            # report experiment error
+            self._report_experiment_error()
         except Exception:
             LOGGER.debug("Error sending os_packages message", exc_info=True)
+            # report experiment error
+            self._report_experiment_error()
 
     def _process_model_graph_message(self, message):
         # type: (ModelGraphMessage) -> None
@@ -420,8 +508,12 @@ class Streamer(BaseStreamer):
                 exc.response.status_code,
                 exc.response.content,
             )
+            # report experiment error
+            self._report_experiment_error()
         except Exception:
             LOGGER.debug("Error sending model_graph message", exc_info=True)
+            # report experiment error
+            self._report_experiment_error()
 
     def _process_system_details_message(self, message):
         # type: (SystemDetailsMessage) -> None
@@ -445,12 +537,16 @@ class Streamer(BaseStreamer):
             )
         except CometRestApiException as exc:
             LOGGER.debug(
-                MODEL_GRAPH_MSG_SENDING_ERROR,
+                SYSTEM_DETAILS_MSG_SENDING_ERROR,
                 exc.response.status_code,
                 exc.response.content,
             )
+            # report experiment error
+            self._report_experiment_error()
         except Exception:
-            LOGGER.debug("Error sending model_graph message", exc_info=True)
+            LOGGER.debug("Error sending system details message", exc_info=True)
+            # report experiment error
+            self._report_experiment_error()
 
     def _process_cloud_details_message(self, message):
         # type: (CloudDetailsMessage) -> None
@@ -466,8 +562,32 @@ class Streamer(BaseStreamer):
                 exc.response.status_code,
                 exc.response.content,
             )
+            # report experiment error
+            self._report_experiment_error()
         except Exception:
             LOGGER.debug("Error sending cloud details message", exc_info=True)
+            # report experiment error
+            self._report_experiment_error()
+
+    def _process_file_name_message(self, message):
+        # type: (FileNameMessage) -> None
+        try:
+            self.rest_api_client.set_experiment_filename(
+                experiment_key=self.experiment_key,
+                filename=message.file_name,
+            )
+        except CometRestApiException as exc:
+            LOGGER.debug(
+                FILENAME_DETAILS_MSG_SENDING_ERROR,
+                exc.response.status_code,
+                exc.response.content,
+            )
+            # report experiment error
+            self._report_experiment_error()
+        except Exception:
+            LOGGER.debug("Error sending file name message", exc_info=True)
+            # report experiment error
+            self._report_experiment_error()
 
     def wait_for_finish(self):
         """Blocks the experiment from exiting until all data was sent to server
@@ -497,7 +617,25 @@ class Streamer(BaseStreamer):
         self.file_upload_manager.close()
 
         # Send all remained parameters from the batch if any
-        self.parameters_batch.accept(self._send_ws_message, unconditional=True)
+        if not self.parameters_batch.empty():
+            if not self.parameters_batch.accept(
+                self._send_ws_message, unconditional=True
+            ):
+                LOGGER.warning(
+                    "Failed to send parameters batch at the end of experiment"
+                )
+                # report experiment error
+                self._report_experiment_error()
+
+        # send all remained metrics from the batch if any
+        if self.message_batch_enabled:
+            if not self.message_batch_metrics.empty():
+                if not self.message_batch_metrics.accept(
+                    self._send_metric_messages_batch, unconditional=True
+                ):
+                    LOGGER.warning(
+                        "Failed to send metrics batch at the end of experiment"
+                    )
 
         if not self.file_upload_manager.all_done():
             monitor = FileUploadManagerMonitor(self.file_upload_manager)
@@ -525,6 +663,8 @@ class Streamer(BaseStreamer):
                     STREAMER_WAIT_FOR_FINISH_FAILED % (remaining, remaining_upload)
                 ),
             )
+            # report experiment error
+            self._report_experiment_error()
 
             return False
 
@@ -532,12 +672,17 @@ class Streamer(BaseStreamer):
 
         return True
 
+    def _report_experiment_error(self):
+        self.rest_api_client.update_experiment_error_status(
+            experiment_key=self.experiment_key, is_alive=True, has_error=True
+        )
+
     def _is_msg_queue_empty(self):
         finished = self.messages.empty()
 
         if finished is False:
             LOGGER.debug(
-                "Message queue not empty, %d messages, closed %s",
+                "WebSocketMessage queue not empty, %d messages, closed %s",
                 self.messages.qsize(),
                 self.closed,
             )
@@ -565,7 +710,7 @@ def compact_json_dump(data, fp):
 class OfflineStreamer(BaseStreamer):
     """
     This class extends threading.Thread and provides a simple concurrent queue
-    and an async service that pulls data from the queue and sends it to the server.
+    and an async service that pulls data from the queue and writes it to the file.
     """
 
     def __init__(self, tmp_dir, initial_offset):
@@ -573,7 +718,9 @@ class OfflineStreamer(BaseStreamer):
         self.daemon = True
         self.tmp_dir = tmp_dir
 
-        self.file = open(os.path.join(self.tmp_dir, "messages.json"), "wb")
+        self.file = open(
+            os.path.join(self.tmp_dir, OFFLINE_EXPERIMENT_MESSAGES_JSON_FILE_NAME), "wb"
+        )
 
     def _write(self, json_line_message):
         # type: (Dict[str, Any]) -> None
@@ -607,8 +754,12 @@ class OfflineStreamer(BaseStreamer):
                         self._process_upload_message(message)
                     elif isinstance(message, UploadInMemoryMessage):
                         self._process_upload_in_memory_message(message)
-                    elif isinstance(message, Message):
+                    elif isinstance(message, WebSocketMessage):
                         self._process_ws_message(message)
+                    elif isinstance(message, MetricMessage):
+                        self._process_metric_message(message)
+                    elif isinstance(message, ParameterMessage):
+                        self._process_parameter_message(message)
                     elif isinstance(message, OsPackagesMessage):
                         self._process_os_package_message(message)
                     elif isinstance(message, ModelGraphMessage):
@@ -619,6 +770,8 @@ class OfflineStreamer(BaseStreamer):
                         self._process_cloud_details_message(message)
                     elif isinstance(message, RemoteAssetMessage):
                         self._process_remote_asset_message(message)
+                    elif isinstance(message, FileNameMessage):
+                        self._process_file_name_message(message)
                     else:
                         raise ValueError("Unkown message type %r", message)
 
@@ -654,7 +807,7 @@ class OfflineStreamer(BaseStreamer):
         message.file_path = basename(tmpfile.name)
 
         msg_json = message.repr_json()
-        data = {"type": "file_upload", "payload": msg_json}
+        data = {"type": UploadFileMessage.type, "payload": msg_json}
         self._write(data)
 
     def _process_upload_in_memory_message(self, message):
@@ -679,46 +832,67 @@ class OfflineStreamer(BaseStreamer):
 
         return self._process_upload_message(new_message)
 
-    def _process_ws_message(self, message):
-        # type: (Message) -> None
+    def _process_metric_message(self, message):
+        # type: (MetricMessage) -> None
         msg_json = message._non_null_dict()
 
-        data = {"type": "ws_msg", "payload": msg_json}
+        data = {"type": MetricMessage.type, "payload": msg_json}
+        self._write(data)
+
+    def _process_parameter_message(self, message):
+        # type: (ParameterMessage) -> None
+        msg_json = message._non_null_dict()
+
+        data = {"type": ParameterMessage.type, "payload": msg_json}
+        self._write(data)
+
+    def _process_ws_message(self, message):
+        # type: (WebSocketMessage) -> None
+        msg_json = message._non_null_dict()
+
+        data = {"type": WebSocketMessage.type, "payload": msg_json}
         self._write(data)
 
     def _process_os_package_message(self, message):
         # type: (OsPackagesMessage) -> None
         msg_json = message._non_null_dict()
 
-        data = {"type": "os_packages", "payload": msg_json}
+        data = {"type": OsPackagesMessage.type, "payload": msg_json}
         self._write(data)
 
     def _process_model_graph_message(self, message):
         # type: (ModelGraphMessage) -> None
         msg_json = message._non_null_dict()
 
-        data = {"type": "graph", "payload": msg_json}
+        data = {"type": ModelGraphMessage.type, "payload": msg_json}
         self._write(data)
 
     def _process_system_details_message(self, message):
         # type: (SystemDetailsMessage) -> None
         msg_json = message._non_null_dict()
 
-        data = {"type": "system_details", "payload": msg_json}
+        data = {"type": SystemDetailsMessage.type, "payload": msg_json}
         self._write(data)
 
     def _process_cloud_details_message(self, message):
         # type: (CloudDetailsMessage) -> None
         msg_json = message._non_null_dict()
 
-        data = {"type": "cloud_details", "payload": msg_json}
+        data = {"type": CloudDetailsMessage.type, "payload": msg_json}
         self._write(data)
 
     def _process_remote_asset_message(self, message):
         # type: (RemoteAssetMessage) -> None
         msg_json = message._non_null_dict()
 
-        data = {"type": "remote_file", "payload": msg_json}
+        data = {"type": RemoteAssetMessage.type, "payload": msg_json}
+        self._write(data)
+
+    def _process_file_name_message(self, message):
+        # type: (FileNameMessage) -> None
+        msg_json = message._non_null_dict()
+
+        data = {"type": FileNameMessage.type, "payload": msg_json}
         self._write(data)
 
     def wait_for_finish(self):
