@@ -20,7 +20,7 @@ import os
 import os.path
 import shutil
 
-import six
+import six.moves
 from everett.manager import (
     NO_VALUE,
     ConfigDictEnv,
@@ -32,6 +32,7 @@ from everett.manager import (
     parse_bool,
 )
 
+from ._jupyter import _in_colab_environment
 from ._logging import _get_comet_logging_config
 from ._typing import Any, Dict, List, Optional, Tuple, Union
 from .exceptions import InvalidRestAPIKey
@@ -40,8 +41,17 @@ from .logging_messages import (
     API_KEY_IS_INVALID,
     API_KEY_IS_NOT_SET,
     API_KEY_IS_VALID,
+    IN_COLAB_WITHOUT_DRIVE_DIRECTORY1,
+    IN_COLAB_WITHOUT_DRIVE_DIRECTORY2,
+)
+from .utils import (
+    clean_string,
+    get_api_key_from_user,
+    get_root_url,
+    is_interactive,
+    log_once_at_level,
+    sanitize_url,
 )
-from .utils import get_api_key_from_user, is_interactive, log_once_at_level
 
 try:  # everett version 1.0.1 or greater
     from everett.ext.inifile import ConfigIniEnv
@@ -79,12 +89,30 @@ DEFAULT_ARTIFACT_DOWNLOAD_TIMEOUT = 3 * 60 * 60  # 3 hours
 
 DEFAULT_INITIAL_DATA_LOGGER_JOIN_TIMEOUT = 5 * 60
 
-DEFAULT_WS_JOIN_TIMEOUT = 10
+DEFAULT_WS_JOIN_TIMEOUT = 30
 
 DEFAULT_PARAMETERS_UPDATE_INTERVAL_SECONDS = 60
 
+DEFAULT_WAIT_FOR_FINISH_SLEEP_INTERVAL = 15
+
 DEFAULT_OFFLINE_DATA_DIRECTORY = ".cometml-runs"
 
+COLAB_DRIVE_MOUNT = "/content/drive/MyDrive/"
+
+DEFAULT_POOL_RATIO = 4
+
+MAX_POOL_SIZE = 64
+
+MESSAGE_BATCH_USE_COMPRESSION_DEFAULT = True
+
+MESSAGE_BATCH_METRIC_INTERVAL_SECONDS = 2
+
+MESSAGE_BATCH_METRIC_MAX_BATCH_SIZE = 1000
+
+OFFLINE_EXPERIMENT_MESSAGES_JSON_FILE_NAME = "messages.json"
+
+OFFLINE_EXPERIMENT_JSON_FILE_NAME = "experiment.json"
+
 
 def get_global_experiment():
     global experiment
@@ -96,6 +124,37 @@ def set_global_experiment(new_experiment):
     experiment = new_experiment
 
 
+def _clean_config_path(file_path):
+    # type: (str) -> str
+    """Apply the usual path cleaning function for config paths"""
+    return os.path.abspath(os.path.expanduser(file_path))
+
+
+def _config_path_from_directory(directory):
+    # type: (str) -> str
+    return _clean_config_path(os.path.join(directory, ".comet.config"))
+
+
+def _get_default_config_path():
+    # type: () -> str
+    config_home = os.environ.get("COMET_CONFIG")
+    if config_home is not None:
+        if config_home is not None and os.path.isdir(config_home):
+            config_home = _config_path_from_directory(config_home)
+
+        return _clean_config_path(config_home)
+
+    elif _in_colab_environment():
+        if os.path.isdir(COLAB_DRIVE_MOUNT):
+            return _config_path_from_directory(COLAB_DRIVE_MOUNT)
+        else:
+            LOGGER.warning(IN_COLAB_WITHOUT_DRIVE_DIRECTORY1)
+            LOGGER.warning(IN_COLAB_WITHOUT_DRIVE_DIRECTORY2)
+            return _config_path_from_directory("~")
+    else:
+        return _config_path_from_directory("~")
+
+
 def parse_str_or_identity(_type):
     def parse(value):
         if not isinstance(value, str):
@@ -149,6 +208,7 @@ class Config(object):
         self.config_map = config_map
         self.override = {}  # type: Dict[str, Any]
         self.backend_override = ConfigDictEnv({})
+
         config_override = os.environ.get("COMET_INI")
         if config_override is not None:
             log_once_at_level(
@@ -156,12 +216,17 @@ class Config(object):
             )
         else:
             config_override = os.environ.get("COMET_CONFIG")
+
+        if config_override is not None and os.path.isdir(config_override):
+            config_override = _config_path_from_directory(config_override)
+
         self.manager = ConfigManager(
             [  # User-defined overrides
                 ConfigOSEnv(),
                 ConfigEnvFileEnv(".env"),
                 ConfigIniEnv(config_override),
                 ConfigIniEnv("./.comet.config"),
+                ConfigIniEnv("/content/drive/MyDrive/.comet.config"),
                 ConfigIniEnv("~/.comet.config"),
                 # Comet-defined overrides
                 self.backend_override,
@@ -480,7 +545,7 @@ class Config(object):
         print("    %d) Operating System Variable" % n)
         n += 1
         for path in ["./.env", "~/.comet.config", "./.comet.config"]:
-            path = os.path.abspath(os.path.expanduser(path))
+            path = _clean_config_path(path)
             if os.path.exists(path):
                 print("    %d) %s" % (n, path))
                 n += 1
@@ -591,7 +656,15 @@ class Config(object):
                     "config setting %r failed with value %r: %s", setting, value, reason
                 )
 
-    def save(self, directory="./", save_all=False, force=False, **kwargs):
+    def save(
+        self,
+        directory=None,
+        filename=None,
+        save_all=False,
+        force=False,
+        _prompt_user_confirmation=False,
+        **kwargs
+    ):
         """
         Save the settings to .comet.config (default) or
         other path/filename. Defaults are commented out.
@@ -602,21 +675,38 @@ class Config(object):
             force: force the file to save if it exists; else don't overwrite
             kwargs: key=value pairs to save
         """
-        directory = os.path.expanduser(directory)
-        filename = os.path.abspath(os.path.join(directory, ".comet.config"))
+        if directory is not None:
+            filename = _config_path_from_directory(directory)
+
+        if filename is None:
+            filename = _get_default_config_path()
+
         if os.path.isfile(filename):
             if not force:
                 LOGGER.error(
                     "'%s' exists and force is not True; refusing to overwrite", filename
                 )
                 return
-            else:  # try to make a backup copy
-                try:
-                    shutil.copyfile(filename, filename + ".bak")
-                except Exception:
+            else:
+                # ASK the user and try to make a backup copy
+                if _prompt_user_confirmation:
+                    overwrite = _confirm_user_config_file_overwriting(filename)
+                else:
+                    # Assume user consent with only the force flag
+                    overwrite = True
+
+                if overwrite:
+                    try:
+                        shutil.copyfile(filename, filename + ".bak")
+                    except Exception:
+                        LOGGER.warning(
+                            "Unable to make a backup of config file", exc_info=True
+                        )
+                else:
                     LOGGER.warning(
-                        "Unable to make a backup of config file", exc_info=True
+                        "User refused to overwrite config file %r, aborting", filename
                     )
+                    return
 
         print('Saving config to "%s"...' % filename, end="")
         with io.open(filename, "w", encoding="utf-8") as ini_file:
@@ -694,10 +784,15 @@ CONFIG_MAP = {
     "comet.offline_directory": {"type": str, "default": DEFAULT_OFFLINE_DATA_DIRECTORY},
     "comet.git_directory": {"type": str},
     "comet.offline_sampling_size": {"type": int, "default": 15000},
-    "comet.url_override": {"type": str, "default": "https://www.comet.ml/clientlib/"},
-    "comet.optimizer_url": {"type": str, "default": "https://www.comet.ml/optimizer/"},
+    "comet.url_override": {
+        "type": str,
+        "default": "https://www.comet-ml.com/clientlib/",
+    },
+    "comet.optimizer_url": {
+        "type": str,
+        "default": None,
+    },
     "comet.ws_url_override": {"type": str, "default": None},
-    "comet.predictor_url": {"type": str, "default": "https://predictor.comet.ml/"},
     "comet.experiment_key": {"type": str},
     "comet.project_name": {"type": str},
     "comet.workspace": {"type": str},
@@ -738,7 +833,6 @@ CONFIG_MAP = {
         "type": int,
         "default": DEFAULT_ARTIFACT_DOWNLOAD_TIMEOUT,
     },
-    "comet.timeout.predictor": {"type": int, "default": 60},
     # HTTP Allow header
     "comet.allow_header.name": {"type": str},
     "comet.allow_header.value": {"type": str},
@@ -757,6 +851,9 @@ CONFIG_MAP = {
     "comet.override_feature.artifacts": {
         "type": bool
     },  # Leave feature toggle default to None
+    "comet.override_feature.sdk_metrics_http_batching": {
+        "type": bool
+    },  # Leave feature toggle default to None
     # Experiment log controls:
     "comet.auto_log.cli_arguments": {"type": bool},
     "comet.auto_log.code": {"type": bool},
@@ -789,12 +886,38 @@ CONFIG_MAP = {
     "comet.auto_log.metric_step_rate": {"type": int, "default": 10},
     "comet.auto_log.co2": {"type": bool},
     "comet.auto_log.tfma": {"type": bool, "default": False},
+    "comet.distributed_node_identifier": {"type": str},
     # Internals:
     "comet.internal.reporting": {"type": bool, "default": True},
-    "comet.internal.file_upload_worker_ratio": {"type": int, "default": 4},
+    "comet.internal.file_upload_worker_ratio": {
+        "type": int,
+        "default": DEFAULT_POOL_RATIO,
+    },
+    "comet.internal.worker_count": {"type": int},
+    "comet.internal.check_tls_certificate": {"type": bool, "default": True},
+    "comet.internal.sentry_dsn": {"type": str},
+    "comet.internal.sentry_debug": {"type": bool, "default": False},
     # Deprecated:
     "comet.display_summary": {"type": bool, "default": None},
     "comet.auto_log.weights": {"type": bool, "default": None},
+    # Related to `comet_ml.start`
+    "comet.resume_strategy": {"type": str, "default": None},
+    "comet.offline": {"type": bool, "default": False},
+    # Error tracking
+    "comet.error_tracking.enable": {"type": bool, "default": True},
+    # Related to message batch processing
+    "comet.message_batch.use_compression": {
+        "type": bool,
+        "default": MESSAGE_BATCH_USE_COMPRESSION_DEFAULT,
+    },
+    "comet.message_batch.metric_interval": {
+        "type": float,
+        "default": MESSAGE_BATCH_METRIC_INTERVAL_SECONDS,
+    },
+    "comet.message_batch.metric_max_size": {
+        "type": int,
+        "default": MESSAGE_BATCH_METRIC_MAX_BATCH_SIZE,
+    },
 }
 
 
@@ -854,11 +977,17 @@ def get_ws_url(ws_server_from_backend, config):
     """Allow users to override the WS url from the backend using the usual
     config mechanism
     """
-    ws_server_from_config = config["comet.ws_url_override"]
-    if ws_server_from_config is None:
-        return ws_server_from_backend
-    else:
-        return ws_server_from_config
+    ws_server = config.get_string(
+        None, "comet.ws_url_override", default=ws_server_from_backend
+    )
+    # Ws server can be None if the backend stop returning it or when used in `comet_check` for example
+    if ws_server is None:
+        return ws_server
+
+    return sanitize_url(
+        ws_server,
+        ending_slash=False,
+    )
 
 
 def get_previous_experiment(previous_experiment, config):
@@ -868,14 +997,15 @@ def get_previous_experiment(previous_experiment, config):
         return previous_experiment
 
 
-def save(directory="~/", save_all=False, force=False, **settings):
+def save(directory=None, save_all=False, force=False, **settings):
     """
     An easy way to create a config file.
 
     Args:
         directory: str (optional), location to save the
             .comet.config file. Typical values are "~/" (home)
-            and "./" (current directory). Default is "~/"
+            and "./" (current directory). Default is "~/" or
+            COMET_CONFIG, if set
         save_all: bool (optional). If True, will create
             entries for all items that are configurable
             with their defaults. Default is False
@@ -911,15 +1041,14 @@ def save(directory="~/", save_all=False, force=False, **settings):
     cfg.save(directory, save_all=save_all, force=force)
 
 
-def _api_key_save(directory, api_key):
+def _api_key_save(config_path, api_key):
+    # type: (str, str) -> None
     """
     Low-level function to only change the api_key
     of a .comet.config file in the home directory.
     """
     from configobj import ConfigObj
 
-    config_path = os.path.expanduser(os.path.join(directory, ".comet.config"))
-
     if os.path.exists(config_path):
         config = ConfigObj(config_path)
         try:
@@ -938,7 +1067,7 @@ def _api_key_save(directory, api_key):
     LOGGER.info("Comet API key saved in %s", config_path)
 
 
-def init(directory="~/", **settings):
+def init(directory=None, **settings):
     # type: (Optional[str], **Any) -> None
     """
     An easy, safe, interactive way to set and save your settings.
@@ -949,7 +1078,8 @@ def init(directory="~/", **settings):
     Args:
         directory: str (optional), location to save the
             .comet.config file. Typical values are "~/" (home)
-            and "./" (current directory). Default is "./"
+            and "./" (current directory). Default is "~/" or
+            COMET_CONFIG, if set.
         settings: any valid setting and value
 
     Valid settings include:
@@ -983,19 +1113,15 @@ def init(directory="~/", **settings):
     >>> comet_ml.init(project_name='colab')
     ```
     """
-    _init(directory=directory, **settings)
+    _init(directory=directory, settings=settings)
 
 
-def _init(directory="~/", __should_prompt_function=None, **settings):
-    # type: (Optional[str], Any, **Any) -> None
-    # We only save the api_key if given by parameter, or
-    # by getpass. We don't save it if in the environment
-    from .api import API
+def _init_get_api_key(prompt_user, settings, config):
+    # type: (bool, Optional[Dict[str, Any]], Config) -> Tuple[Optional[str], bool]
 
-    if __should_prompt_function is None:
-        __should_prompt_function = is_interactive
+    if settings is None:
+        settings = {}
 
-    config = get_config()
     need_to_save = False
     if "api_key" in settings:
         api_key = settings.pop("api_key", None)
@@ -1005,39 +1131,133 @@ def _init(directory="~/", __should_prompt_function=None, **settings):
         api_key = config.get_string(None, "comet.api_key")
 
     if api_key is None:
-        if __should_prompt_function():
+        if prompt_user:
             api_key = get_api_key_from_user()
             if api_key:
                 need_to_save = True
 
-    failed = False
+    return api_key, need_to_save
+
+
+def _check_api_key_validity(api_key):
+    # type: (str) -> None
+    from .api import API
+
+    API(api_key=api_key).get_workspaces()
+
+
+def _init(directory=None, should_prompt_user=None, settings=None):
+    # type: (Optional[str], Optional[bool], Optional[Dict[str, Any]]) -> None
+    # We only save the api_key if given by parameter, or
+    # by getpass. We don't save it if in the environment
+
+    config = get_config()
+
+    if should_prompt_user is None:
+        should_prompt_user = is_interactive()
+
+    api_key, need_to_save = _init_get_api_key(should_prompt_user, settings, config)
+
     if api_key is None:
         LOGGER.info(API_KEY_IS_NOT_SET)
     else:
         try:
-            API(api_key=api_key).get_workspaces()
+            _check_api_key_validity(api_key)
         except InvalidRestAPIKey:
-            failed = True
             LOGGER.error(
                 API_KEY_IS_INVALID,
                 config.get_string(None, "comet.url_override"),
                 exc_info=True,
             )
+            return None
         except Exception:
-            failed = True
             LOGGER.error(API_KEY_CHECK_FAILED, exc_info=True)
+            return None
+
+        # From here we know that the API Key is valid for the configured Comet installation
+        LOGGER.info(API_KEY_IS_VALID)
+
+        if need_to_save:
+            try:
+                if directory is not None:
+                    config_path = _config_path_from_directory(directory)
+                else:
+                    config_path = _get_default_config_path()
+
+                _api_key_save(config_path, api_key)
+            except Exception:
+                LOGGER.warning("Unable to save Comet API key to disk", exc_info=True)
+                return None
+
+    if settings:
+        # Set in environment to save:
+        config._set_settings(settings, environ=True)
+
+
+def init_onprem(force=False):
+    # type: (bool) -> None
+
+    if force:
+        for var in ["COMET_OPTIMIZER_URL", "COMET_URL_OVERRIDE", "COMET_API_KEY"]:
+            if var in os.environ:
+                del os.environ[var]
+
+    config = get_config()
+
+    client_url = config.get_string(None, "comet.url_override")
+
+    root_url = sanitize_url(get_root_url(client_url))
+
+    if root_url == "https://www.comet-ml.com/" or force:
+        LOGGER.info(
+            "For help on these settings, please see: https://www.comet.ml/docs/onprem/"
+        )
+
+        client_url = _input_user("Please enter your onprem COMET_URL_OVERRIDE: ")
+
+        # Set the environment variables, config.save will save them in the local config file
+        os.environ["COMET_URL_OVERRIDE"] = client_url
+
+    # We currently force the user confirmation but it might hang
+    # if not in a TTY
+    config.save(_prompt_user_confirmation=True, force=force)
+
+    # Set api key if necessary, and save it too
+    try:
+        api_key, need_to_save = _init_get_api_key(
+            prompt_user=True,
+            settings={},
+            config=config,
+        )
+
+        if api_key is None:
+            LOGGER.info(API_KEY_IS_NOT_SET)
         else:
+            _check_api_key_validity(api_key)
+
+            # From here we know that the API Key is valid for the configured Comet installation
             LOGGER.info(API_KEY_IS_VALID)
 
             if need_to_save:
-                try:
-                    _api_key_save(directory, api_key)
-                except Exception:
-                    failed = True
-                    LOGGER.warning(
-                        "Unable to save Comet API key to disk", exc_info=True
-                    )
+                config_path = _get_default_config_path()
 
-    if settings and not failed:
-        # Set in environment to save:
-        config._set_settings(settings, environ=True)
+                _api_key_save(config_path, api_key)
+    except Exception:
+        # TODO: Add more specific error message for individual part of the try block
+        raise Exception(
+            "invalid url or api key; use comet_ml.init_onprem(force=True) or `comet init --onprem --force` to reset"
+        )
+
+
+def _confirm_user_config_file_overwriting(filename):
+    # type: (str) -> bool
+    prompt = "Are you sure you want to overwrite your %r file? [y/n] " % filename
+    if _input_user(prompt).lower().startswith("y"):
+        return True
+    else:
+        return False
+
+
+def _input_user(prompt):
+    """Independent function to apply clean_string to all responses + make mocking easier"""
+    return clean_string(six.moves.input(prompt))
