@@ -19,10 +19,12 @@ from collections import defaultdict
 
 from ._typing import Any, Dict, Iterable, List, Optional, Set, Tuple, Union
 from .convert_utils import (
+    HAS_NUMPY,
     convert_tensor_to_numpy,
     convert_to_bytes,
     convert_to_list,
     convert_to_scalar,
+    numpy,
 )
 from .logging_messages import (
     CONFUSION_MATRIX_ERROR_RESULTING_LENGTH,
@@ -32,7 +34,7 @@ from .logging_messages import (
     CONFUSION_MATRIX_EXAMPLE_NONE,
     CONFUSION_MATRIX_INDEX_TO_EXAMPLE_ERROR,
 )
-from .utils import HAS_NUMPY, is_list_like, log_once_at_level, numpy
+from .utils import is_list_like, log_once_at_level
 
 LOGGER = logging.getLogger(__name__)
 LOG_ONCE_CACHE = set()  # type: Set[str]
