@@ -73,10 +73,6 @@ class FakeStd(object):
         self.__wrapped__ = original
         self.handler = handler
 
-        # Only Python2 std* has writelines method
-        if not hasattr(original, "writelines"):
-            del self.writelines
-
     def write(self, line):
         """
         Overrides the default IO write(). Writes to console + queue.
@@ -113,6 +109,19 @@ class FakeStd(object):
     def close(self):
         self.flush()
 
+    def __getattribute__(self, attr):
+        # Some std wrappers don't expose writelines, so we need to do extra work to detect such cases and not expose it either
+        if attr == "writelines":
+            self_wrapped = object.__getattribute__(self, "__wrapped__")
+            try:
+                _ = getattr(self_wrapped, "writelines")
+                # Wrapped defines it, returns overridden method
+                return object.__getattribute__(self, "writelines")
+            except AttributeError:
+                raise
+
+        return object.__getattribute__(self, attr)
+
     def __getattr__(self, attr):
         # We cannot use wrapt in Python2 around sys.stdout/sys.stderr because
         # they are files and we cannot set custom attributes to files in Python
