@@ -90,6 +90,14 @@ class ProjectNameEmpty(CometException):
         self.args = tuple()
 
 
+class ProjectNameIsTooLong(CometException):
+    def __init__(self, err_msg):
+        super(CometException, self).__init__()
+        # Use backend provided err_msg
+        self.log_message = err_msg
+        self.args = tuple()
+
+
 class InvalidOfflineDirectory(CometException):
     def __init__(self, directory, reason):
         self.directory = directory
@@ -147,6 +155,16 @@ class ExperimentAlreadyUploaded(CometException):
         return self.msg % self.experiment_id
 
 
+class OfflineExperimentUploadFailed(CometException):
+    """Exception to mark issues with uploading of the offline experiments"""
+
+    def __init__(self, msg):
+        self.msg = msg
+
+    def __str__(self):
+        return self.msg
+
+
 class AssetIsTooBig(CometException):
     msg = "Asset %r size %d is greater than the upload limit %d"
 
@@ -342,3 +360,4 @@ EXPERIMENT_ALREADY_EXISTS = 90999
 INVALID_OPTIMIZATION_ID = 5124123
 NO_PROJECT_NAME_SPECIFIED = 98219
 NON_EXISTING_TEAM = 90219
+PROJECT_NAME_TOO_LONG = 1341241
