@@ -12,11 +12,15 @@
 #  permission of Comet ML Inc.
 # *******************************************************
 import logging
-import tempfile
 from os.path import basename, join
 
 from .._typing import Optional
-from ..config import get_config, get_global_experiment, set_global_experiment
+from ..config import (
+    get_api_key,
+    get_config,
+    get_global_experiment,
+    set_global_experiment,
+)
 from ..experiment import BaseExperiment
 from ..logging_messages import (
     MLFLOW_NESTED_RUN_UNSUPPORTED,
@@ -55,7 +59,7 @@ def _create_experiment(experiment_name=None):
 
     global PROJECT_NAME
 
-    api_key = get_config("comet.api_key")
+    api_key = get_api_key(None, get_config())
 
     if api_key:
         from comet_ml import Experiment
@@ -64,27 +68,19 @@ def _create_experiment(experiment_name=None):
 
         exp = Experiment(api_key, project_name=PROJECT_NAME)
     else:
-        offline_dir = get_config("comet.offline_directory")
-
         LOGGER.info(MLFLOW_OFFLINE_EXPERIMENT_FALLBACK)
-
-        if not offline_dir:
-            offline_dir = tempfile.mkdtemp()
-
         LOGGER.debug(
-            "Creating an offline Experiment with project name %r and offline directory %r",
-            PROJECT_NAME,
-            offline_dir,
+            "Creating an offline Experiment with project name %r.", PROJECT_NAME
         )
 
         exp = OfflineExperiment(
-            project_name=PROJECT_NAME, offline_directory=offline_dir
+            project_name=PROJECT_NAME,
         )
 
     if experiment_name:
         exp.set_name(experiment_name)
 
-    # Mark the experiment as created implicitely from MLFlow logger
+    # Mark the experiment as created implicitly from MLFlow logger
     exp.log_other("Created from", "MLFlow auto-logger")
 
     return exp
@@ -199,8 +195,13 @@ def mlflow_start_run(
 
     # Detect continuing runs
     new_run_status = getattr(getattr(return_value, "_info", None), "status", None)
+    new_run_id = getattr(getattr(return_value, "_info", None), "run_id", None)
 
-    if new_run_status == "FINISHED":
+    # The latest MLFlow version, on resuming of the existing experiment, sets the status to the RunStatus.RUNNING
+    # and run_id to the run_id value that was returned by previous run.
+    if new_run_status == "FINISHED" or (
+        new_run_status == "RUNNING" and run_id == new_run_id
+    ):
         LOGGER.info(MLFLOW_RESUMED_RUN)
 
     if IMPLICIT_START_RUN:
