@@ -16,8 +16,8 @@
 import json
 import logging
 
+from ..convert_utils import data_to_fp
 from ..monkey_patching import check_module
-from ..utils import data_to_fp
 
 LOGGER = logging.getLogger(__name__)
 SLICING_METRICS_COUNT = 0
