@@ -42,7 +42,7 @@ INVALID_REST_API_KEY = (
 
 METRIC_ARRAY_WARNING = (
     "Cannot safely convert %r object to a scalar value, using its string"
-    " representation for logging."
+    " representation for logging. Resulting string might be invalid"
 )
 
 PARSING_ERR_MSG = """We failed to parse your parameter configuration file.
@@ -77,11 +77,11 @@ LOG_FIGURE_TOO_BIG = "Figure number %d is bigger than the upload limit, %s > %s"
 LOG_AUDIO_TOO_BIG = "Audio %s is bigger than the upload limit, %s > %s"
 
 NATIVE_STD_WRAPPER_NOT_AVAILABLE = (
-    "Native output logging mode is not available, fallbacking on basic output logging"
+    "Native output logging mode is not available, falling back to basic output logging"
 )
 
 UNKOWN_STD_WRAPPER_SPEC = (
-    "Unknown output logging mode: %s, fallbacking on basic output logging"
+    "Unknown output logging mode: %s, falling back to basic output logging"
 )
 
 EXPERIMENT_LIVE = "Experiment is live on comet.ml %s\n"
@@ -92,6 +92,8 @@ OFFLINE_SENDER_STARTS = "Starting the upload of the experiment"
 
 OFFLINE_SENDER_ENDS = "The offline experiment has been uploaded on comet.ml %s\n"
 
+OFFLINE_SENDER_ENDS_PROCESSING = "Offline Experiment will now be processed---may take some time to appear on the Comet dashboard."
+
 OFFLINE_DATA_DIR_DEFAULT_WARNING = (
     "Using %r path as offline directory. Pass 'offline_directory' parameter into constructor "
     "or set the 'COMET_OFFLINE_DIRECTORY' environment variable to manually choose where to store offline "
@@ -103,14 +105,66 @@ OFFLINE_DATA_DIR_FAILED_WARNING = (
     "Reason: %s"
 )
 
+OFFLINE_UPLOADING_EXPERIMENT_FILE_PROMPT = "Attempting to upload %r..."
+
+OFFLINE_SUCCESS_UPLOADED_EXPERIMENTS = "Number of uploaded experiments: %d"
+
+OFFLINE_FAILED_UPLOADED_EXPERIMENTS = "Number of failed experiment uploads: %d"
+
+OFFLINE_UPLOAD_FAILED_UNEXPECTED_ERROR = "Upload failed for %r; unknown error"
+
+OFFLINE_UPLOAD_FAILED_INVALID_API_KEY = (
+    "Upload failed because of invalid Comet API key; please set COMET_API_KEY"
+)
+
+OFFLINE_UPLOADS_FAILED_DUE_TIMEOUT = (
+    "Scheduled uploads failed due to timeout, remaining uploads: %d"
+)
+
 ADD_TAGS_ERROR = "Failed to add tag(s) %s to the experiment\n"
 
 ADD_SYMLINK_ERROR = "Failed to create symlink to project:%s for experiment\n"
 
 OFFLINE_EXPERIMENT_INVALID_WS_MSG = "An invalid message has been detected"
 
+OFFLINE_EXPERIMENT_INVALID_METRIC_MSG = "An invalid metric message has been detected."
+
 OFFLINE_EXPERIMENT_INVALID_UPLOAD_MSG = "An invalid upload message has been detected"
 
+OFFLINE_EXPERIMENT_INVALID_PARAMETER_MSG = (
+    "An invalid parameter message has been detected"
+)
+
+OFFLINE_EXPERIMENT_INVALID_OS_PACKAGES_MSG = (
+    "An invalid OS packages message has been detected"
+)
+
+OFFLINE_EXPERIMENT_INVALID_GRAPH_MSG = "An invalid graph message has been detected"
+
+OFFLINE_EXPERIMENT_INVALID_SYSTEM_DETAILS_MSG = (
+    "An invalid system details message has been detected"
+)
+
+OFFLINE_EXPERIMENT_INVALID_CLOUD_DETAILS_MSG = (
+    "An invalid cloud details message has been detected"
+)
+
+OFFLINE_EXPERIMENT_INVALID_FILE_NAME_MSG = (
+    "An invalid file name message has been detected"
+)
+
+OFFLINE_EXPERIMENT_ALREADY_UPLOADED = "Upload failed for %r; experiment was already uploaded, you can re-upload it by using the --force-reupload flag"
+
+OFFLINE_AT_LEAST_ONE_EXPERIMENT_UPLOAD_FAILED = "At least one experiment failed to upload, check COMET ERROR messages above for more information."
+
+OFFLINE_EXPERIMENT_CREATION_WORKSPACE_OVERRIDDEN_PARAMETER = "Using workspace '%(creation_workspace)s', set by parameter rather than workspace '%(metadata_workspace)s' set in the Offline Experiment Archive"
+
+OFFLINE_EXPERIMENT_CREATION_PROJECT_NAME_OVERRIDDEN_PARAMETER = "Using project_name '%(creation_project_name)s', set by parameter rather than project_name '%(metadata_project_name)s' set in the Offline Experiment Archive"
+
+OFFLINE_EXPERIMENT_CREATION_WORKSPACE_OVERRIDDEN_CONFIG = "Using workspace '%(creation_workspace)s', set in configuration rather than workspace '%(metadata_workspace)s' set in the Offline Experiment Archive"
+
+OFFLINE_EXPERIMENT_CREATION_PROJECT_NAME_OVERRIDDEN_CONFIG = "Using project_name '%(creation_project_name)s', set in configuration rather than project_name '%(metadata_project_name)s' set in the Offline Experiment Archive"
+
 EXPERIMENT_INVALID_STEP = "Passed step value %r is not a number, ignoring it"
 
 EXPERIMENT_INVALID_EPOCH = "Passed epoch value %r is not a number, ignoring it"
@@ -147,15 +201,17 @@ WS_SSL_ERROR_MSG = (
     "https://www.comet.ml/docs/python-sdk/warnings-errors/"
 )
 
-OFFLINE_EXPERIMENT_ALREADY_UPLOADED = (
-    "Experiment %r was already uploaded, you can re-upload it by using the"
-    " --force-reupload flag"
-)
-
 SEND_NOTIFICATION_FAILED = (
     "Error sending a notification, make sure you have opted-in for notifications"
 )
 
+EXPERIMENT_MARK_AS_STARTED_FAILED = (
+    "Failed to report experiment status (experiment started)"
+)
+EXPERIMENT_MARK_AS_ENDED_FAILED = (
+    "Failed to report experiment status (experiment ended)"
+)
+
 GET_CALLBACK_FAILURE = "Failure to add Comet SDK callback to %s, the run will likely miss parameters and/out metrics"
 
 WS_ON_OPEN_MSG = "WS Socket connection open"
@@ -168,10 +224,20 @@ LOG_DATASET_ERROR = "Failed to create dataset hash"
 
 OS_PACKAGE_MSG_SENDING_ERROR = "Error sending os_packages message, got %d %r"
 
+METRICS_BATCH_MSG_SENDING_ERROR = "Error sending metrics batch message, got %d %r"
+
 MODEL_GRAPH_MSG_SENDING_ERROR = "Error sending model graph message, got %d %r"
 
+SYSTEM_DETAILS_MSG_SENDING_ERROR = (
+    "Error sending model system details message, got %d %r"
+)
+
 CLOUD_DETAILS_MSG_SENDING_ERROR = "Error sending cloud details messages, got %d %r"
 
+FILENAME_DETAILS_MSG_SENDING_ERROR = (
+    "Error sending file name details messages, got %d %r"
+)
+
 OPTIMIZER_KWARGS_CONSTRUCTOR_DEPRACETED = "Passing Experiment through Optimizer constructor is deprecated; pass them to Optimizer.get_experiments or Optimizer.next"
 
 BACKEND_VERSION_CHECK_ERROR = "Failed to check backend version at URL: %r"
@@ -285,6 +351,8 @@ API_KEY_CHECK_FAILED = "Unable to verify Comet API key at this time"
 
 JUPYTER_NEEDS_END = "As you are running in a Jupyter environment, you will need to call `experiment.end()` when finished to ensure all metrics and code are logged before exiting."
 
+PYDEV_NEEDS_END = "As you are running in a Pydev Console, you will need to call `experiment.end()` when finished to ensure all metrics and code are logged before exiting."
+
 ARTIFACT_VERSION_CREATED_WITH_PREVIOUS = (
     "Artifact %r version %s created (previous was: %s)"
 )
@@ -343,6 +411,16 @@ ARTIFACT_DOWNLOAD_FINISHED = "Artifact '%s/%s:%s' has been successfully download
 
 ARTIFACT_ASSET_DOWNLOAD_FAILED = "Asset %r of the artifact '%s/%s:%s' download failed"
 
+ARTIFACT_ASSET_DOWNLOAD_FAILED_REPR = "Cannot download Asset %r for Artifact %s"
+
+ARTIFACT_ASSET_DOWNLOAD_FAILED_WITH_ERROR = (
+    "Cannot download synced Remote Asset with logical path %r, original error: '%s'"
+)
+
+ARTIFACT_ASSET_WRITE_ERROR = (
+    "Cannot write Asset %r on disk path %r, check logs for details"
+)
+
 FILE_DOWNLOAD_MANAGER_COMPLETED = "All files downloaded"
 
 FILE_DOWNLOAD_MANAGER_MONITOR_FIRST_MESSAGE = (
@@ -362,3 +440,83 @@ INITIAL_DATA_LOGGER_FLUSHING_EXPERIMENT_INTERRUPTED_BY_USER = "Experiment was in
 INITIAL_DATA_LOGGER_FLUSHING_FAILED = (
     "Unexpected failure while waiting for the initial data logger to be flushed."
 )
+
+LOG_GIT_METADATA_ERROR = "Failed to log git metadata"
+
+LOG_GIT_PATCH_ERROR = "Failed to log git patch"
+
+GIT_REPO_NOT_FOUND = "Couldn't find a Git repository in %r nor in any parent directory. You can override where Comet is looking for a Git Patch by setting the configuration `COMET_GIT_DIRECTORY`"
+
+GIT_LOGGING_ERROR = "Error logging git-related information"
+
+REPORTING_ERROR = "Failing to report %s"
+
+OFFLINE_EXPERIMENT_NAME_ACCESS = "Experiment Name is generated at upload time for Offline Experiments unless set explicitly with Experiment.set_name"
+
+IN_COLAB_WITHOUT_DRIVE_DIRECTORY1 = (
+    "running in Google Colab, but can't find mounted drive. Using HOME instead"
+)
+IN_COLAB_WITHOUT_DRIVE_DIRECTORY2 = (
+    "if drive is mounted, set COMET_CONFIG to save config there"
+)
+
+START_PARAMETERS_DONT_MATCH_EXISTING_EXPERIMENT = "An experiment already exists %r but all its parameters don't match comet_ml.start, closing it and recreating one"
+
+START_RUNNING_EXPERIMENT_IGNORED_PARAMETER = "Parameter '%s' with value '%s' was ignored while checking if Experiment instance %r matched the start parameters"
+
+LOG_COLAB_NOTEBOOK_ERROR = "Couldn't retrieve Google Colab notebook content"
+
+CONVERT_TABLE_INVALID_FORMAT = "Tabular filename must end with '.tsv' or '.csv', not %r"
+
+CONVERT_DATAFRAME_INVALID_FORMAT = (
+    "Tabular dataframe filename must end with 'json', 'csv', 'md', or 'html', not %r"
+)
+
+LOG_TABLE_NONE_VALUES = "Either tabular_data or a valid file-path are required when calling log_table; ignoring"
+
+LOG_TABLE_FILENAME_AND_HEADERS = (
+    "Headers are ignored when calling log_table with a filename"
+)
+
+DATAFRAME_CONVERSION_ERROR = "dataframe conversion to %r failed; ignored"
+
+METRIC_NONE_WARNING = "The given value of the metric %s was None; ignoring"
+
+LATE_IMPORT_DEGRADED_MODE = "Comet has disabled auto-logging functionality as it has been imported after the following ML modules: %s. Metrics and hyperparameters can still be logged using comet_ml.log_metrics() and comet_ml.log_parameters()"
+
+LOG_CONDA_ENV_ERROR = "Unknown error exporting current conda environment"
+LOG_CONDA_ENV_RETURN_CODE = "Error exporting current conda environment"
+
+LOG_CONDA_PACKAGES_ERRORS = "Unknown error retrieving Conda package as an explicit file"
+LOG_CONDA_PACKAGES_RETURN_CODE = "Error retrieving Conda package as an explicit file"
+
+LOG_CONDA_INFO_ERRORS = "Unknown error retrieving Conda information"
+LOG_CONDA_INFO_RETURN_CODE = "Error retrieving Conda information"
+
+OPTIMIZER_COULDNT_END = "Optimizer couldn't end"
+
+FAILED_LOG_EMBEDDING_GROUPS = "Couldn't log embedding groups"
+
+FAILED_LOG_IPYTHON_NOTEBOOK = "Couldn't log ipython notebook"
+
+COULDNT_COMPLETE_ATEXIT_UNREGISTER = "Couldn't complete atexit unregister"
+
+KUBEFLOW_LOGGER_ERROR = "Unkown error logging Kubeflow data in comet_ml.integration.kubeflow.initialize_comet_logger, experiment will be tagged with %r"
+
+KUBEFLOW_LOGGER_IMPORT_ERROR = "python package 'kfp' is required to use comet_ml.integration.kubeflow.initialize_comet_logger, make sure to install a version compatible with your environment"
+
+FAILED_TO_ADD_ARTIFACT_REMOTE_SYNC_ASSET = (
+    "Failed to add remote synced asset with URI '%s'. Fallback to not synced mode."
+)
+
+SYNC_MODE_IS_NOT_SUPPORTED_FOR_STRING_REMOTE_ARTIFACT = "Synced mode is not supported for string remote artifact assets (URI '%s'). Fallback to not synced mode."
+
+FAILED_TO_ADD_ARTIFACT_S3_ASSETS_TOO_MANY_KEYS = "Failed to add remote artifact assets from S3 bucket '%s', the number of files in the bucket exceeds max_synced_objects: %d"
+FAILED_TO_ADD_ARTIFACT_GS_ASSETS_TOO_MANY_KEYS = "Failed to add remote artifact assets from GS bucket '%s', the number of files in the bucket exceeds max_synced_objects: %d"
+
+BOTO3_IMPORT_FAILED = "boto3 is not installed or cannot be imported and is needed to sync s3:// remote asset urls."
+
+
+GCP_CLIENT_IMPORT_FAILED = "GCP client is not installed or cannot be imported and is needed to sync gs:// remote asset urls."
+
+UNSUPPORTED_URI_SYNCED_REMOTE_ASSET = "Unsupported URI '%s' for synced remote asset."
