@@ -12,13 +12,47 @@
 #  the express permission of Comet ML Inc.
 # *******************************************************
 import json
+import re
+from abc import ABCMeta, abstractproperty
 
-from ._typing import Any, Callable, Dict, List, MemoryUploadable, Optional
+from ._typing import Any, Callable, Dict, List, MemoryUploadable, Optional, Type
+from .convert_utils import fix_special_floats
 from .json_encoder import NestedEncoder
-from .utils import fix_special_floats, local_timestamp
+from .utils import local_timestamp
 
 
 class BaseMessage(object):
+    __metaclass__ = ABCMeta
+
+    @abstractproperty
+    def type(self):
+        pass
+
+    @staticmethod
+    def translate_message_class__to_handler_name(class_name):
+        """
+        This function turns the message class name to the equivalent function in the Message class.
+        """
+        class_name_splitted = re.findall("[A-Z][^A-Z]*", class_name)
+        class_name_splitted_lowered = [word.lower() for word in class_name_splitted]
+        method_name = "_".join(class_name_splitted_lowered[:-1])
+        return "set_" + method_name
+
+    @classmethod
+    def create(cls, context, config, **kwargs):
+        if config["comet.override_feature.use_http_messages"]:
+            return cls(**kwargs)
+        else:
+            return cls.create_websocket_message(context, **kwargs)
+
+    @classmethod
+    def create_websocket_message(self, context, **kwargs):
+        message = WebSocketMessage(context=context)
+        method_name = self.translate_message_class__to_handler_name(self.__name__)
+        method = getattr(message, method_name)
+        method(**kwargs)
+        return message
+
     def repr_json(self):
         return {
             key: value
@@ -26,6 +60,10 @@ class BaseMessage(object):
             if (not key.startswith("_"))
         }
 
+    def repr_json_batch(self):
+        """To be defined by subclasses to return JSON dictionary representation to be used for batching"""
+        pass
+
     def to_json(self):
         json_re = json.dumps(
             self.repr_json(), sort_keys=True, indent=4, cls=NestedEncoder
@@ -41,23 +79,24 @@ class BaseMessage(object):
 
 
 class CloseMessage(BaseMessage):
-    """A special messag indicating Streamer to ends and exit"""
+    """A special message indicating Streamer to ends and exit"""
+
+    type = "close"
 
     pass
 
 
-class Message(BaseMessage):
+class WebSocketMessage(BaseMessage):
     """
     A bean used to send messages to the server over websockets.
     """
 
+    type = "ws_msg"
+
     def __init__(self, context=None):
         self.local_timestamp = local_timestamp()
 
         # The following attributes are optional
-        self.metric = None
-        self.param = None
-        self.params = None
         self.graph = None
         self.code = None
         self.stdout = None
@@ -87,35 +126,12 @@ class Message(BaseMessage):
     def set_installed_packages(self, val):
         self.installed_packages = val
 
-    def set_os_packages(self, val):
-        self.os_packages = val
-
-    def set_metric(self, name, value, step=None, epoch=None):
-        safe_value = fix_special_floats(value)
-        self.metric = {
-            "metricName": name,
-            "metricValue": safe_value,
-            "step": step,
-            "epoch": epoch,
-        }
-
     def set_html(self, value):
         self.html = value
 
-    def set_htmlOverride(self, value):
+    def set_html_override(self, value):
         self.htmlOverride = value
 
-    def set_param(self, name, value, step=None):
-        safe_value = fix_special_floats(value)
-        self.param = {"paramName": name, "paramValue": safe_value, "step": step}
-
-    def set_params(self, name, values, step=None):
-        safe_values = list(map(fix_special_floats, values))
-        self.params = {"paramName": name, "paramValue": safe_values, "step": step}
-
-    def set_graph(self, graph):
-        self.graph = graph
-
     def set_code(self, code):
         self.code = code
 
@@ -127,11 +143,8 @@ class Message(BaseMessage):
         self.stdout = line
         self.stderr = True
 
-    def set_env_details(self, details):
-        self.env_details = details
-
-    def set_filename(self, fname):
-        self.fileName = fname
+    def set_file_name(self, file_name):
+        self.fileName = file_name
 
     def set_gpu_static_info(self, info):
         self.gpu_static_info = info
@@ -155,6 +168,8 @@ class Message(BaseMessage):
 
 
 class UploadFileMessage(BaseMessage):
+    type = "file_upload"
+
     def __init__(
         self,
         file_path,  # type: str
@@ -198,6 +213,8 @@ class UploadFileMessage(BaseMessage):
 
 
 class UploadInMemoryMessage(BaseMessage):
+    type = "file_upload"
+
     def __init__(
         self,
         file_like,  # type: MemoryUploadable
@@ -239,6 +256,8 @@ class UploadInMemoryMessage(BaseMessage):
 
 
 class RemoteAssetMessage(BaseMessage):
+    type = "remote_file"
+
     def __init__(
         self,
         remote_uri,  # type: Any
@@ -269,6 +288,8 @@ class RemoteAssetMessage(BaseMessage):
 
 
 class OsPackagesMessage(BaseMessage):
+    type = "os_packages"
+
     def __init__(self, os_packages):
         # type: (List[str]) -> None
         self.os_packages = os_packages
@@ -278,6 +299,8 @@ class OsPackagesMessage(BaseMessage):
 
 
 class ModelGraphMessage(BaseMessage):
+    type = "graph"
+
     def __init__(self, graph):
         # type: (str) -> None
         self.graph = graph
@@ -287,6 +310,8 @@ class ModelGraphMessage(BaseMessage):
 
 
 class SystemDetailsMessage(BaseMessage):
+    type = "system_details"
+
     def __init__(
         self,
         command,  # type: str
@@ -326,6 +351,8 @@ class SystemDetailsMessage(BaseMessage):
 
 
 class CloudDetailsMessage(BaseMessage):
+    type = "cloud_details"
+
     def __init__(
         self,
         provider,  # type: str
@@ -338,3 +365,141 @@ class CloudDetailsMessage(BaseMessage):
 
     def _non_null_dict(self):
         return self.__dict__
+
+
+class ParameterMessage(BaseMessage):
+
+    """The Message type to encapsulate named parameter value.
+    The parameter value can be either float or the list of floats."""
+
+    type = "parameter_msg"
+
+    def __init__(self, context=None, timestamp=None):
+        # type: (Optional[str], Optional[int]) -> None
+        if timestamp is None:
+            timestamp = local_timestamp()
+
+        self.context = context
+        self.local_timestamp = timestamp
+
+        # The following attributes are optional
+        self.param = None  # type: Optional[Dict[str, Any]]
+        self.params = None  # type: Optional[Dict[str, Any]]
+
+    def set_param(self, name, value, step=None):
+        # type: (str, Any, Optional[int]) -> None
+        safe_value = fix_special_floats(value)
+        self.param = {"paramName": name, "paramValue": safe_value, "step": step}
+
+    def set_params(self, name, values, step=None):
+        # type: (str, List[Any], Optional[int]) -> None
+        safe_values = list(map(fix_special_floats, values))
+        self.params = {"paramName": name, "paramValue": safe_values, "step": step}
+
+    def get_param_name(self):
+        # type: () -> Optional[str]
+        """Returns the name of the parameter associated with this message."""
+        if self.param:
+            return self.param["paramName"]
+        elif self.params:
+            return self.params["paramName"]
+        else:
+            return None
+
+    @classmethod
+    def deserialize(cls, message_dict):
+        # type: (Type['ParameterMessage'], Dict[str, Any]) -> 'ParameterMessage'
+        """Recreate a ParameterMessage from its Dict representation"""
+        parameter_message = cls(
+            context=message_dict.get("context", None),
+            timestamp=message_dict.get("local_timestamp", None),
+        )
+
+        if message_dict.get("param", None):
+            parameter_message.set_param(
+                message_dict["param"]["paramName"],
+                message_dict["param"]["paramValue"],
+                step=message_dict["param"]["step"],
+            )
+        elif message_dict.get("params", None):
+            parameter_message.set_params(
+                message_dict["params"]["paramName"],
+                message_dict["params"]["paramValue"],
+                step=message_dict["params"]["step"],
+            )
+        else:
+            # TODO: WHAT TO DO?
+            pass
+
+        return parameter_message
+
+
+class MetricMessage(BaseMessage):
+    type = "metric_msg"
+    metric_key = "metric"
+
+    """The Message type to encapsulate named metric value."""
+
+    def __init__(self, context=None, timestamp=None):
+        # type: (Optional[str], Optional[int]) -> None
+        self.context = context
+        self.metric = None  # type: Optional[Dict[str, Any]]
+        if timestamp is None:
+            timestamp = local_timestamp()
+
+        self.local_timestamp = timestamp
+
+    def set_metric(self, name, value, step=None, epoch=None):
+        safe_value = fix_special_floats(value)
+        self.metric = {
+            "metricName": name,
+            "metricValue": safe_value,
+            "step": step,
+            "epoch": epoch,
+        }
+
+    @classmethod
+    def deserialize(cls, message_dict):
+        # type: (Type['MetricMessage'], Dict[str, Any]) -> MetricMessage
+        """Recreate a MetricMessage from its Dict representation"""
+        metric_message = cls(
+            context=message_dict.get("context", None),
+            timestamp=message_dict.get("local_timestamp", None),
+        )
+
+        if message_dict.get(cls.metric_key, None):
+            metric_message.set_metric(
+                message_dict[cls.metric_key]["metricName"],
+                message_dict[cls.metric_key]["metricValue"],
+                step=message_dict[cls.metric_key].get("step", None),
+                epoch=message_dict[cls.metric_key].get("epoch", None),
+            )
+        else:
+            raise ValueError("no metric message data found")
+
+        return metric_message
+
+    def repr_json_batch(self):
+        # type: () -> Dict[str, Any]
+        repr_dict = dict(self.metric)
+        repr_dict["context"] = self.context
+        repr_dict["timestamp"] = self.local_timestamp
+        return repr_dict
+
+    def __str__(self):
+        return "MetricMessage: %s" % self.repr_json_batch()
+
+
+class FileNameMessage(BaseMessage):
+    type = "file_name"
+
+    def __init__(
+        self,
+        file_name,  # type: str
+    ):
+        # type: (...) -> None
+
+        self.file_name = file_name
+
+    def _non_null_dict(self):
+        return self.__dict__
