@@ -23,7 +23,6 @@ from .connection import get_optimizer_api
 from .exceptions import InvalidOptimizerParameters, OptimizerException
 from .experiment import BaseExperiment
 from .logging_messages import OPTIMIZER_KWARGS_CONSTRUCTOR_DEPRACETED
-from .predictor import Predictor
 
 LOGGER = logging.getLogger(__name__)
 
@@ -126,8 +125,11 @@ class Optimizer(object):
                 optimizer config filename.
             trials: int (optional, default 1) number of trials per parameter value set
             verbose: int (optional, default 1) level of details to show; 0 means quiet
-            experiment_class: string or callable (optional) the class to use when
-                when generating experiments.
+            experiment_class: string (optional). Supported values are "Experiment" (the
+                default) to use online Experiments or "OfflineExperiment" to use offline
+                Experiments. It can also be a callable (a function or a method) that returns an
+                instance of Experiment, OfflineExperiment, ExistingExperiment or
+                ExistingOfflineExperiment.
 
         See above for examples.
         """
@@ -147,9 +149,10 @@ class Optimizer(object):
         self.experiment_kwargs = kwargs
         self.config = get_config()
         self.api_key = get_api_key(api_key, self.config)
-        self._api = get_optimizer_api(self.api_key)
+        self._api = get_optimizer_api(
+            self.api_key,
+        )
         self.id = None
-        self.predictor_spec = None
         if os.environ.get("COMET_OPTIMIZER_ID") is not None:
             self.id = os.environ.get("COMET_OPTIMIZER_ID")
             if verbose > 0:
@@ -180,8 +183,6 @@ class Optimizer(object):
         status = self.status()
         if "code" in status:
             raise OptimizerException("invalid optimizer instance status: %s" % status)
-        if "predictor" in status and status["predictor"]:
-            self.predictor_spec = self._fill_defaults_predictor(status["predictor"])
         elif verbose > 0:
             LOGGER.info("Using optimizer config: %s", status)
 
@@ -364,6 +365,7 @@ class Optimizer(object):
         objective = "minimum"
         version = status["version"]
         score = None
+        name = status["name"]
         ## Find correct, min/max metric
         spec_metric = experiment._summary.get("metrics", metric)
         if spec_metric:
@@ -392,19 +394,15 @@ class Optimizer(object):
         experiment.log_other("optimizer_process", os.getpid())
         experiment.log_other("optimizer_count", count)
         experiment.log_other("optimizer_objective", objective)
+        experiment.log_other("optimizer_name", name)
         experiment.log_other(
             "optimizer_parameters",
             json.dumps(
                 {key: experiment.params.get(key, None) for key in status["parameters"]}
             ),
         )
-        if self.predictor_spec is not None:
-            if self.predictor_spec["mode"] == "local":
-                return False  # don't force experiment to wait until uploading is done
-            else:
-                return True  # force experiment to wait until uploading is done
-        else:
-            return False  # don't force experiment to wait until uploading is done
+
+        return False  # don't force experiment to wait until uploading is done
 
     def _load(self, optimizer_file, trials=None, verbose=1):
         """
@@ -450,26 +448,6 @@ class Optimizer(object):
                 if "scalingType" not in parameters[param]:
                     parameters[param]["scalingType"] = "uniform"
 
-    def _fill_defaults_predictor(self, predictor_spec):
-        """
-        Fill in defaults for predictor, if given.
-        """
-        if predictor_spec is not None:
-            # Keep in sync with Predictor.__init__ default values
-            defaults = {
-                "loss_name": "loss",
-                "mode": "global",
-                "n_init_experiments": 0,
-                "patience": 10,
-                "start_step": 10,
-                "threshold": 0.1,
-                "time_interval": 300,
-            }
-            defaults.update(predictor_spec)
-            return defaults
-        else:
-            return None
-
     def _set(
         self,
         algorithm,
@@ -478,7 +456,6 @@ class Optimizer(object):
         trials=1,
         spec=None,
         verbose=1,
-        predictor=None,
     ):
         """
         Create or use existing Optimizer.
@@ -505,10 +482,6 @@ class Optimizer(object):
             spec = {}
 
         spec = self._complete_spec(spec, algorithm)
-        # Save the predictor spec:
-        self.predictor_spec = self._fill_defaults_predictor(predictor)
-        if self.predictor_spec:
-            LOGGER.info("Using Predictor in Optimizer with: %s", self.predictor_spec)
 
         if self.id is None:
             ## /create
@@ -518,7 +491,6 @@ class Optimizer(object):
                 "parameters": converted_parameters,
                 "trials": trials,
                 "spec": spec,
-                "predictor": self.predictor_spec,
             }
             results = self._api.post_request("create", json=payload)
             if results["code"] == 200:
@@ -620,33 +592,6 @@ class Optimizer(object):
 
         exp._set_optimizer(self, pid, trial, count)
 
-        if self.predictor_spec:
-            LOGGER.debug("Setting predictor in experiment: %s", self.predictor_spec)
-
-            if isinstance(exp, OfflineExperiment):
-                mode = "local"
-            else:
-                # This should be at least global as it is set in Optimizer._fill_defaults_predictor
-                mode = self.predictor_spec["mode"]
-
-            # Old version of the Python SDK used to store predictor_spec with an
-            # empty optimizer_id, which would leads to TypeError: type object
-            # got multiple values for keyword argument 'optimizer_id'
-            # Also pop mode as it is passed explicitely
-            predictor_spec = self.predictor_spec.copy()
-            predictor_spec.pop("optimizer_id", None)
-            predictor_spec.pop("mode", None)
-
-            try:
-                exp.set_predictor(
-                    Predictor(exp, optimizer_id=self.id, mode=mode, **predictor_spec)
-                )
-            except Exception:
-                LOGGER.warning(
-                    "Failure to create Predictor, experiment will run without Predictor",
-                    exc_info=True,
-                )
-
         exp.log_parameters(parameters)
         # Log optimizer static information:
         exp.log_other("optimizer_id", self.id)
