@@ -50,7 +50,22 @@ import zipfile
 from collections import defaultdict
 
 from .._typing import Dict, Optional
-from ..config import get_config
+from ..config import (
+    OFFLINE_EXPERIMENT_JSON_FILE_NAME,
+    OFFLINE_EXPERIMENT_MESSAGES_JSON_FILE_NAME,
+    get_config,
+)
+from ..messages import (
+    CloudDetailsMessage,
+    FileNameMessage,
+    ModelGraphMessage,
+    OsPackagesMessage,
+    ParameterMessage,
+    RemoteAssetMessage,
+    SystemDetailsMessage,
+    UploadFileMessage,
+    WebSocketMessage,
+)
 from ..utils import format_bytes
 
 LOGGER = logging.getLogger("comet_ml")
@@ -148,12 +163,15 @@ class ArchivedExperiment(object):
         """
         Process the message JSON file.
         """
-        with self.archive.open("messages.json") as fp:
+        with self.archive.open(OFFLINE_EXPERIMENT_MESSAGES_JSON_FILE_NAME) as fp:
             for line in fp:
                 message = json.loads(line)
-                if message["type"] == "ws_msg":
+                if (
+                    message["type"] == WebSocketMessage.type
+                    or message["type"] == ParameterMessage.type
+                ):
                     self.process_payload(message["payload"])
-                elif message["type"] == "file_upload":
+                elif message["type"] == UploadFileMessage.type:
                     payload = message["payload"]
                     file_info = [
                         fi
@@ -168,13 +186,16 @@ class ArchivedExperiment(object):
                     self.update_summary(
                         "data", "file uploads", size, count_only=False, sum_only=True
                     )
-                    self.write("detail", "file_upload", payload["upload_type"], size)
+                    self.write(
+                        "detail", UploadFileMessage.type, payload["upload_type"], size
+                    )
                 elif message["type"] in [
-                    "graph",
-                    "os_packages",
-                    "system_details",
-                    "cloud_details",
-                    "remote_file",
+                    RemoteAssetMessage.type,
+                    ModelGraphMessage.type,
+                    OsPackagesMessage.type,
+                    SystemDetailsMessage.type,
+                    CloudDetailsMessage.type,
+                    FileNameMessage.type,
                 ]:
                     continue
                 else:
@@ -304,7 +325,7 @@ class ArchivedExperiment(object):
         """
         Process the experiment JSON file.
         """
-        with self.archive.open("experiment.json") as fp:
+        with self.archive.open(OFFLINE_EXPERIMENT_JSON_FILE_NAME) as fp:
             line = fp.readline()
             experiment = json.loads(line)
             self.workspace = experiment["workspace"] or "DEFAULT"
