@@ -20,6 +20,8 @@ import argparse
 import logging
 import sys
 
+from comet_ml.exceptions import InvalidAPIKey, OfflineExperimentUploadFailed
+from comet_ml.logging_messages import OFFLINE_UPLOAD_FAILED_INVALID_API_KEY
 from comet_ml.offline import main_upload
 
 LOGGER = logging.getLogger("comet_ml")
@@ -37,10 +39,34 @@ def get_parser_arguments(parser):
         const=True,
         default=False,
     )
+    parser.add_argument(
+        "--workspace",
+        dest="override_workspace",
+        help="upload all offline experiment archives to that workspace instead of the original workspace",
+        action="store",
+    )
+    parser.add_argument(
+        "--project-name",
+        dest="override_project_name",
+        help="upload all offline experiment archives to that project instead of the original project",
+        action="store",
+    )
 
 
 def upload(args, rest=None):
-    main_upload(args.archives, args.force_reupload)
+    try:
+        main_upload(
+            archives=args.archives,
+            force_reupload=args.force_reupload,
+            override_workspace=args.override_workspace,
+            override_project_name=args.override_project_name,
+        )
+    except InvalidAPIKey:
+        LOGGER.error(OFFLINE_UPLOAD_FAILED_INVALID_API_KEY, exc_info=True)
+        sys.exit(1)
+    except OfflineExperimentUploadFailed as ex:
+        LOGGER.error(ex.msg, exc_info=True)
+        sys.exit(1)
 
 
 def main(args):
