@@ -126,7 +126,7 @@ class Summary(object):
                     minmax += 1
                     details[detail_name] = (key_summary["min"], key_summary["max"])
                 else:
-                    if key_summary["value"]:
+                    if key_summary["value"] is not None:
                         details[detail_name] = key_summary["value"]
                     else:  # counts, and maybe size
                         if key_summary["size"]:
